/* -*- C++ -*- */

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "SOAP" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 *
 *
 *
 * @author Roshan Weerasuriya (roshan@jkcs.slt.lk, roshan@opensource.lk)
 *
 */

#include <ctype.h>
#include <axis/client/Call.h>
#include <axis/wsdd/WSDDDeployment.h>
#include "AdminClientHelper.h"

int processDeployWSDD(char* pchWSDDPath);
int processUnDeployWSDD(char* pchWSDDPath);
void printMessage();

const AxisChar* pAchServiceName;
const AxisChar* pAchLibName;
list<AxisString> lstAllowedMethods;
//string* sArrAllowedMethods;
string sArrAllowedMethods[10];
int iAllowdMethodsArraySize=0;
char* pchMethodName = "";
ADMIN_CLIENT_METHOD eADMIN_CLIENT_METHOD;
char* pchMessage = "";
WSDDDeployment* pWSDDDeployment = NULL;

int main(int argc, char *argv[]) 
{
	string sURI = "";
	int nPort = 0;

	if (argc < 3) {
		printf("Usage: %s <WSDD File Path> <Server> <Port>", argv[0]);	
		exit(1);
	}

	string sServer = argv[2];
	string sPort = (argc == 4)?argv[3]:"80";

	if (0 == (nPort = atoi(sPort.c_str()))) 
	{
		printf("\nError : Invalid port\n");
		exit(1);
	}

 	int nStatus;
	int ret = 0;
	char* chWSDDPath = "";	

	Call* m_pCall;
	m_pCall = new Call();

	/*
	 *Process the client side deploy or undeploy WSDD and extract the required details
	 */

	do {

		chWSDDPath = argv[1];

		pWSDDDeployment= new WSDDDeployment();
		if (pWSDDDeployment->LoadWSDD(chWSDDPath) != AXIS_SUCCESS) 
		{
			pchMessage = "Failed processing the given WSDD. Please check whether \n1) The wsdd exists in the given path\
					\n2) The wsdd is correct etc\n";
			break;
		}

		eADMIN_CLIENT_METHOD = (pWSDDDeployment->GetDeploymentType() == DT_DEPLOYMENT)? AC_DEPLOY:AC_UNDEPLOY;

		if (eADMIN_CLIENT_METHOD == AC_DEPLOY) {
			if ( processDeployWSDD(chWSDDPath) != AXIS_SUCCESS ) {
				printMessage();
				break;
			}
		} else if (eADMIN_CLIENT_METHOD == AC_UNDEPLOY) {
			if ( processUnDeployWSDD(chWSDDPath) != AXIS_SUCCESS ) {
				printMessage();
				break;
			}
		}

		/* The protocol can be taken from the binding section in the WSDL */ //????? 
		m_pCall->SetProtocol(APTHTTP);
		/* The value for SOAPAction is specified in the binding section of the WSDL */ //????? 
		m_pCall->SetHeader("SOAPAction", "WebserviceDeployer");
		/* End point URI too can be taken from WSDL */ //????? 
		sURI = "http://" + sServer + ":" + sPort + "/axis/WebserviceDeployer";
		m_pCall->SetEndpointURI(sURI.c_str());

		/* Following will establish the connections with the server too */
		if (AXIS_SUCCESS != m_pCall->Initialize()) 
			return ret;
		m_pCall->SetSOAPVersion(SOAP_VER_1_1);
		/* This should be done before adding parameters or return type */
		
		if (eADMIN_CLIENT_METHOD == AC_DEPLOY) {
			m_pCall->SetOperation("deploy", "http://www.opensource.lk/");
		
			m_pCall->AddParameter(pAchServiceName, "ServiceName", XSD_STRING);
			m_pCall->AddParameter(pAchLibName, "LibName", XSD_STRING);

			Axis_Array* structAxis_Array= (Axis_Array*) malloc(sizeof(Axis_Array));

			structAxis_Array->m_Array = sArrAllowedMethods;
			structAxis_Array->m_Size = iAllowdMethodsArraySize;
			m_pCall->AddParameter(structAxis_Array, XSD_STRING, "AllowedMethods");
		} else if (eADMIN_CLIENT_METHOD == AC_UNDEPLOY) {
			m_pCall->SetOperation("undeploy", "http://www.opensource.lk/");
		
			m_pCall->AddParameter(pAchServiceName, "ServiceName", XSD_STRING);
		}
		
		m_pCall->SetReturnType(XSD_INT);

		
		nStatus = m_pCall->Invoke();

		if (AXIS_SUCCESS == nStatus)
		{
			m_pCall->GetResult()->GetInt(&ret);
	#ifdef _DEBUG
			printf("%s%d\n", "Got The result = ", ret);
	#endif
			if (ret == 1) {
				if (eADMIN_CLIENT_METHOD == AC_DEPLOY) {
					printf("AXIS_SUCCESS : Deployment is SUCCESSFULL");
				} else if (eADMIN_CLIENT_METHOD == AC_UNDEPLOY) {
					printf("AXIS_SUCCESS : UnDeployment is SUCCESSFULL");
				}
			} else {
				if (eADMIN_CLIENT_METHOD == AC_DEPLOY) {
					printf("UNSUCCESS : Deployment is UNSUCCESSFULL");
				} else if (eADMIN_CLIENT_METHOD == AC_UNDEPLOY) {
					printf("UNSUCCESS : UnDeployment is UNSUCCESSFULL");
				}
			}
		}

		/* Following will close the connection with the server too */
		m_pCall->UnInitialize();

	} while(0);

	if (pWSDDDeployment) delete pWSDDDeployment;
	delete m_pCall;

	return 0;
}

int processDeployWSDD(char* pchWSDDPath) {

	/*
	 *WSDD processing code comes here
	 */
	int iStatus = AXIS_FAIL;

	do {
		const WSDDServiceMap* pWSDDServiceMap= pWSDDDeployment->GetWSDDServiceMap();
		const WSDDServiceMap::const_iterator itCurrService = pWSDDServiceMap->begin();	
		
		if (itCurrService != pWSDDServiceMap->end()) {
			const WSDDService* pWSDDService = (*itCurrService).second;
			pAchServiceName = pWSDDService->GetServiceName();
			pAchLibName = pWSDDService->GetLibName();
			lstAllowedMethods = pWSDDService->getAllowedMethods();
			
			printf("pAchServiceName = %s\n", pAchServiceName);
			printf("pAchLibName = %s\n", pAchLibName);
			
			list<AxisString>::iterator iteAllowedMethods = lstAllowedMethods.begin();
			int iTmpCount =0;
			while (iteAllowedMethods != lstAllowedMethods.end()) {	
				/*DEBUG line
				printf("sAllowedMethod = %s\n", (*iteAllowedMethods).c_str());
				*/
				sArrAllowedMethods[iTmpCount] = (*iteAllowedMethods).c_str();
				iTmpCount++;
				iteAllowedMethods++;
			}
			iAllowdMethodsArraySize = iTmpCount;
		}
	
		iStatus = AXIS_SUCCESS;

	} while (0);

	return iStatus;
}

int processUnDeployWSDD(char* pchWSDDPath) {

	/*
	 *WSDD processing code comes here
	 */

	int iStatus = AXIS_FAIL;

	do {
		const WSDDServiceMap* pWSDDServiceMap= pWSDDDeployment->GetWSDDServiceMap();
		const WSDDServiceMap::const_iterator itCurrService = pWSDDServiceMap->begin();	
		
		if (itCurrService != pWSDDServiceMap->end()) {
			const WSDDService* pWSDDService = (*itCurrService).second;
			pAchServiceName = pWSDDService->GetServiceName();
		}

		iStatus = AXIS_SUCCESS;

	} while (0);

	return iStatus;
}

void printMessage() {
	printf("Message : %s", pchMessage);
}