#include "Point.h"

#include <axis/common/IWrapperSoapDeSerializer.h>
#include <axis/common/IWrapperSoapSerializer.h>
#include <axis/common/IParam.h>

//Parameters and wrapper methods to manipulate Point
static const AxisChar* Axis_URI_Point = "http://www.opensource.lk/Point";
static const AxisChar* Axis_TypeName_Point = "Point";

int Axis_DeSerialize_Point(Point* p, IWrapperSoapDeSerializer *pDZ)
{
	p->x = pDZ->GetInt();
	p->y = pDZ->GetInt();
	return pDZ->GetStatus();
}

void* Axis_Create_Point(bool bArray = false, int nSize=0)
{
	if (bArray && (nSize > 0))
		return new Point[nSize];
	else
		return new Point;
}

void Axis_Delete_Point(Point* p, bool bArray = false, int nSize=0)
{
	if (bArray)
		delete [] p;
	else
		delete p;
}

int Axis_Serialize_Point(Point* p, IWrapperSoapSerializer& pSZ, bool bArray = false)
{
	if (bArray)
	{
		pSZ << "<" << Axis_TypeName_Point << ">";
	}
	else
	{
		const AxisChar* sPrefix = pSZ.getNewNamespacePrefix();
		pSZ << "<" << Axis_TypeName_Point << " xsi:type=\"" << sPrefix <<":" 
			<< Axis_TypeName_Point << "\" xmlns:" << sPrefix << "=\"" 
			<< Axis_URI_Point << "\">";
	}
	pSZ << pSZ.SerializeBasicType("x", p->x, XSD_INT);
	pSZ << pSZ.SerializeBasicType("y", p->y, XSD_INT);
	pSZ << "</" << Axis_TypeName_Point << ">";
	return AXIS_SUCCESS;
}

int Axis_GetSize_Point(Point* p)
{
	return sizeof(Point);
}

//Parameters and wrapper methods to manipulate Polygon
static const AxisChar* Axis_URI_Polygon = "http://www.opensource.lk/Polygon";
static const AxisChar* Axis_TypeName_Polygon = "Polygon";

int Axis_DeSerialize_Polygon(Polygon* p, IWrapperSoapDeSerializer *pIWSDZ)
{
	p->name = pIWSDZ->GetString();
	p->points = (PointArray&)pIWSDZ->GetArray((void*)Axis_DeSerialize_Point
		, (void*)Axis_Create_Point, (void*)Axis_Delete_Point
		, (void*)Axis_GetSize_Point, Axis_TypeName_Point, Axis_URI_Point);
	return pIWSDZ->GetStatus();
}

void* Axis_Create_Polygon(bool bArray = false, int nSize=0)
{
	if (bArray && (nSize > 0))
		return new Polygon[nSize];
	else
		return new Polygon;
}

void Axis_Delete_Polygon(Polygon* p, bool bArray = false, int nSize=0)
{
	if (bArray)
	{
		//delete any pointer members or array members of this struct here
		Polygon* pTemp = p;
		for (int x=0; x<nSize; x++)
		{
			Axis_Delete_Point(pTemp->points.m_Array, true, pTemp->points.m_Size);
			pTemp++;
		}
		delete [] p;
	}
	else
	{
		//delete any pointer members or array members of this struct here
		Axis_Delete_Point(p->points.m_Array, true, p->points.m_Size);
		delete p;
	}
}

int Axis_Serialize_Polygon(Polygon* p, IWrapperSoapSerializer& pSZ, bool bArray = false)
{
	if (bArray)
	{
		pSZ << "<" << Axis_TypeName_Polygon << ">";
	}
	else
	{
		const AxisChar* sPrefix = pSZ.getNewNamespacePrefix();
		pSZ << "<" << Axis_TypeName_Polygon << " xsi:type=\"" << sPrefix <<":" 
			<< Axis_TypeName_Polygon << "\" xmlns:" << sPrefix << "=\"" 
			<< Axis_URI_Polygon << "\">";
	}
	pSZ << pSZ.SerializeBasicType("name", p->name, XSD_STRING);
	//Serialize array.
	pSZ.SerializeArray((Axis_Array*)(&p->points), 
		(void*) Axis_Serialize_Point, (void*) Axis_Delete_Point, (void*) Axis_GetSize_Point, 
		Axis_TypeName_Point, Axis_URI_Point, "points");
	pSZ << "</" << Axis_TypeName_Polygon << ">";
	return AXIS_SUCCESS;
}

int Axis_GetSize_Polygon(Polygon* p)
{
	return sizeof(Polygon);
}
