/* -*- C++ -*- */

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "SOAP" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

/*
 *
 *
 * @author Lilantha Darshana (lilantha@virtusa.com)
 *
 */


#if !defined(_AXIS_LIBAXISCPP_HPP)
#define _AXIS_LIBAXISCPP_HPP

#include "AxisCppContentHandler.h"
#include <strstream>

#ifdef LIBAXISCPP_EXPORTS
#define LIBAXIS_DLL_API __declspec(dllexport)
#else
#define LIBAXIS_DLL_API __declspec(dllimport)
#endif


#ifdef __cplusplus
extern "C" {
#endif

static void
jni_throw(JNIEnv* env, const char* exception, const char* msg)
{
    if (env->ExceptionOccurred())
        return;
    jclass jexception = env->FindClass(exception);
    if (env->ExceptionOccurred())
      return;
    if (jexception == NULL)
        env->FatalError(exception);
    env->ThrowNew(jexception, msg);
}


#ifdef __cplusplus
}
#endif

#define JNI_ASSERT(assert, name, msg) \
    do \
    { \
        if (m_pEnv->ExceptionOccurred()) \
            return; \
        if (! assert) \
        { \
            jni_throw(m_pEnv, name, msg); \
            return; \
        } \
    } while (0)




class JNIVector
{
public:
    JNIVector(JNIEnv* p_Env, jobject p_jVector);
	///Destructor
    ~JNIVector();

    char* operator [] (int i) const;
	void push_back(const char* str);
	void clear();
	int  size();

private:

    JNIEnv* m_pEnv;
    jobject m_jVector;
	jmethodID m_jmGet;
	jmethodID m_jmClear;
	jmethodID m_jmAdd;
	jmethodID m_jmSize;
};



class JNIOutputStream : public std::strstreambuf
{
public:

    /// jobject writer must be a java.io.OutputStream object
    JNIOutputStream(JNIEnv* env, jobject stream, unsigned bufsize = 16300);

    virtual ~JNIOutputStream();

protected:

    virtual int overflow(int c);
    virtual int sync();

private:

    JNIEnv* m_pEnv;

    jobject   _output;
    jmethodID _write;
    jmethodID _flush;

    unsigned _bufsize;
    jbyteArray _jbuf;
};


class JNIInputStream
    : public std::strstreambuf
{
public:

    /// jobject writer must be a java.io.InputStream object
    JNIInputStream(JNIEnv* env, jobject stream, unsigned bufsize = 16300);

    virtual ~JNIInputStream();

    int available();

protected:

    virtual int underflow();

private:

    JNIEnv* m_pEnv;

    jobject _input;
    jmethodID _read;
    jmethodID _close;
    jmethodID _available;

    unsigned _bufsize;
    jbyteArray _jbuf;
    char* _buf;
};

#endif //_AXIS_LIBAXISCPP_HPP

