/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "SOAP" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 *
 *
 *
 * @author  Damitha Kumarage (damitha@opensource.lk, damitha@jkcsworld.com)
 *
 */

#include "SimpleAuthorizationHandler.h"
#include <axis/common/ISoapDeSerializer.h>
#include <axis/common/ISoapSerializer.h>
#include <string.h>
#include <fstream>
#include <string>
//#include <axis/common/AxisTrace.h>
#include <axis/common/IMessageData.h>

SimpleAuthorizationHandler::SimpleAuthorizationHandler()
{
  m_pOption = NULL;
  m_sEmpty = "";
}

SimpleAuthorizationHandler::~SimpleAuthorizationHandler()
{

}
// Implementation of BasicHandler interface.
int SimpleAuthorizationHandler::Invoke(IMessageData* md)
{
  try
  {
    //AXISTRACE1("SimpleAuthorizationHandler::Invoke(IMessageData* md)");
    string sValue = GetOption("AllowByDefault");
    //AXISTRACE2("sValue :", sValue.c_str());
    int intIsAllowed =   atoi(sValue.c_str());    
    if(0 == intIsAllowed)
    {
      return AXIS_SUCCESS;
    }
    m_sAuthUser = md->GetUserName();
    //AXISTRACE2("m_sAuthUser:",m_sAuthUser.c_str());
    if(!m_sAuthUser.empty())
    {    
      string sAllowedRolesFileName = GetOption("AllowedRoles");
      //AXISTRACE2("sAllowedRolesFileName:",sAllowedRolesFileName.c_str());
      ifstream fin(sAllowedRolesFileName.c_str());    // open for reading
      string sLine = "";
      char buff[512];
      while (!fin.eof())
      {
        fin.getline(buff, 512);
        sLine = buff;
        m_AllowedRoles.push_back(sLine);
        //AXISTRACE2("line read is:", sLine.c_str());
      }
       
      m_itCurrentRole = m_AllowedRoles.begin();
	    while (m_itCurrentRole != m_AllowedRoles.end())
	    {
        //AXISTRACE2("(*m_itCurrentRole).c_str()", (*m_itCurrentRole).c_str());
        if(strcmp(m_sAuthUser.c_str(),(*m_itCurrentRole).c_str()) == 0)
        {
          //AXISTRACE1("if(strcmp(m_sAuthUser.c_str(),(*m_itCurrentRole).c_str()) == 0)");
          return AXIS_SUCCESS;
        }
			  m_itCurrentRole++;
      }
      //AXISTRACE1("SimpleAuthorizationHandler invoke end");
      fin.close();      
    }
    else
    {
      return AXIS_FAIL;
    }
  }catch(...)
  {
    return AXIS_FAIL;
  }

  return AXIS_SUCCESS;    
}

void SimpleAuthorizationHandler::OnFault(IMessageData* mc)
{

}

const string& SimpleAuthorizationHandler::GetOption(const string& sArg)
{
	map<string, string>::const_iterator it = m_pOption->find(sArg);
	if (it != m_pOption->end())
	{
		return (*it).second;
	}
	return m_sEmpty;
}

void SimpleAuthorizationHandler::SetOptionList(const map<string, string>* OptionList)
{
   m_pOption = OptionList;
}

int SimpleAuthorizationHandler::Init()
{
	//do any initialization, resetting of values

	return AXIS_SUCCESS;
}

int SimpleAuthorizationHandler::Fini()
{
	//do any finalizatoin

	return AXIS_SUCCESS;
}
