//Manually written Wrapper for C web services
//01 September 2003 by Susantha Kumara (susantha@opensource.lk, skumara@virtusa.com)

#include "CServiceWrapper.hpp"

extern "C" int AddInt(int a, int b);
extern "C" double Distance(Point *p1, Point *p2);
extern "C" Point* AddPoint(Point *p1, Point *p2);
extern "C" double Perimeter(const Triangle* pTri);

//Parameters and wrapper methos to manipulate Point
static const AxisChar* Axis_URI_Point = "http://www.opensource.lk/Point";
static const AxisChar* Axis_TypeName_Point = "Point";

int Axis_DeSerialize_Point(Point* p, IWrapperSoapDeSerializer *pDZ)
{
	p->x = pDZ->GetInt();
	p->y = pDZ->GetInt();
	return pDZ->GetStatus();
}

void* Axis_Create_Point(bool bArray = false, int nSize=0)
{
	if (bArray && (nSize > 0))
		return new Point[nSize];
	else
		return new Point;
}

void Axis_Delete_Point(Point* p, bool bArray = false, int nSize=0)
{
	if (bArray)
		delete [] p;
	else
		delete p;
}

int Axis_Serialize_Point(Point* p, IWrapperSoapSerializer& pSZ, bool bArray = false)
{
	if (bArray)
	{
		pSZ << "<" << Axis_TypeName_Point << ">";
	}
	else
	{
		const AxisChar* sPrefix = pSZ.getNewNamespacePrefix();
		pSZ << "<" << Axis_TypeName_Point << " xsi:type=\"" << sPrefix <<":" 
			<< Axis_TypeName_Point << "\" xmlns:" << sPrefix << "=\"" 
			<< Axis_URI_Point << "\">";
	}
	pSZ << pSZ.SerializeBasicType("x", p->x, XSD_INT);
	pSZ << pSZ.SerializeBasicType("y", p->y, XSD_INT);
	pSZ << "</" << Axis_TypeName_Point << ">";
	return AXIS_SUCCESS;
}

int Axis_GetSize_Point(Point* p)
{
	return sizeof(Point);
}

//Parameters and wrapper methods to manipulate Triangle
static const AxisChar* Axis_URI_Triangle = "http://www.opensource.lk/Triangle";
static const AxisChar* Axis_TypeName_Triangle = "Triangle";

int Axis_DeSerialize_Triangle(Triangle* p, IWrapperSoapDeSerializer *pDZ)
{
	p->p1 = (Point*)pDZ->GetObject((void*)Axis_DeSerialize_Point, 
		(void*)Axis_Create_Point, (void*)Axis_Delete_Point, 
		Axis_TypeName_Point, Axis_URI_Point);
	p->p2 = (Point*)pDZ->GetObject((void*)Axis_DeSerialize_Point, 
		(void*)Axis_Create_Point, (void*)Axis_Delete_Point, 
		Axis_TypeName_Point, Axis_URI_Point);
	p->p3 = (Point*)pDZ->GetObject((void*)Axis_DeSerialize_Point, 
		(void*)Axis_Create_Point, (void*)Axis_Delete_Point, 
		Axis_TypeName_Point, Axis_URI_Point);
	return pDZ->GetStatus();
}

void* Axis_Create_Triangle(bool bArray = false, int nSize=0)
{
	if (bArray && (nSize > 0))
		return new Triangle[nSize];
	else
		return new Triangle;
}

void Axis_Delete_Triangle(Triangle* p, bool bArray = false, int nSize=0)
{
	if (bArray)
	{
		//delete any pointer members of this struct here
		Triangle* pTemp = p;
		for (int x=0; x<nSize; x++)
		{
			Axis_Delete_Point(pTemp->p1);
			Axis_Delete_Point(pTemp->p2);
			Axis_Delete_Point(pTemp->p3);
			pTemp++;
		}
		delete [] p;
	}
	else
	{
		//delete any pointer members of this struct here
		Axis_Delete_Point(p->p1);
		Axis_Delete_Point(p->p2);
		Axis_Delete_Point(p->p3);
		delete p;
	}
}

int Axis_Serialize_Triangle(Triangle* p, IWrapperSoapSerializer& pSZ, bool bArray = false)
{
	if (bArray)
	{
		pSZ << "<" << Axis_TypeName_Triangle << ">";
	}
	else
	{
		AxisString sPrefix = pSZ.getNewNamespacePrefix();
		pSZ << "<" << Axis_TypeName_Triangle << " xsi:type=\"" << sPrefix.c_str() <<":" 
			<< Axis_TypeName_Triangle << " xmlns:" << sPrefix.c_str() << "=\"" 
			<< Axis_URI_Triangle << "\">";
	}
	Axis_Serialize_Point(p->p1, pSZ);
	Axis_Serialize_Point(p->p1, pSZ);
	Axis_Serialize_Point(p->p1, pSZ);
	pSZ << "</" << Axis_TypeName_Triangle << ">";
	return AXIS_SUCCESS;
}

int Axis_GetSize_Triangle(Triangle* p)
{
	return sizeof(Triangle);
}

CServiceWrapper::CServiceWrapper()
{
	//does nothing
}

CServiceWrapper::~CServiceWrapper()
{
	//does nothing
}

// Implementation of WrapperClassHandler interfaces.
int CServiceWrapper::Invoke(IMessageData* mc)
{
	IWrapperSoapDeSerializer* pIWSDZ = NULL;
	mc->getSoapDeSerializer(&pIWSDZ);
	if (!pIWSDZ) return AXIS_FAIL;
	const AxisChar* method = pIWSDZ->GetMethodName();
	if (0 == strcmp(method, "Distance"))
		return DistanceWrapped(mc);
	else if (0 == strcmp(method, "AddPoint"))
		return AddPointWrapped(mc);
	else if (0 == strcmp(method, "Add"))
		return AddWrapped(mc);
	else if (0 == strcmp(method, "Perimeter"))
		return PerimeterWrapped(mc);
	else return AXIS_FAIL;
}

void CServiceWrapper::OnFault(IMessageData* pMsg)
{
}

int CServiceWrapper::Init()
{
	return 0;
}

int CServiceWrapper::Fini()
{
	return 0;
}

int CServiceWrapper::DistanceWrapped(IMessageData* mc)
{
	IWrapperSoapSerializer* pIWSSZ = NULL;
	mc->getSoapSerializer(&pIWSSZ);
	if (!pIWSSZ) return AXIS_FAIL;
	IWrapperSoapDeSerializer* pIWSDZ = NULL;
	mc->getSoapDeSerializer(&pIWSDZ);
	if (!pIWSDZ) return AXIS_FAIL;
	pIWSSZ->createSoapMethod("DistanceResponse", pIWSSZ->getNewNamespacePrefix(), "http://www.opensource.lk/CPPService");

	Point* v0 = (Point*)pIWSDZ->GetObject((void*)Axis_DeSerialize_Point, 
		(void*)Axis_Create_Point, (void*)Axis_Delete_Point, 
		Axis_TypeName_Point, Axis_URI_Point);

	Point* v1 = (Point*)pIWSDZ->GetObject((void*)Axis_DeSerialize_Point, 
		(void*)Axis_Create_Point, (void*)Axis_Delete_Point, 
		Axis_TypeName_Point, Axis_URI_Point);

	if (AXIS_SUCCESS != pIWSDZ->GetStatus())
	{
		Axis_Delete_Point(v0);
		Axis_Delete_Point(v1);
		return AXIS_DESERIALIZATION_ERROR;
	}
	//Call actual web service method with appropriate parameters
	double ret = Distance(v0, v1);

	return pIWSSZ->AddOutputParam("DistanceReturn", ret, XSD_DOUBLE);
}

int CServiceWrapper::AddPointWrapped(IMessageData* mc)
{
	IWrapperSoapSerializer* pIWSSZ = NULL;
	mc->getSoapSerializer(&pIWSSZ);
	if (!pIWSSZ) return AXIS_FAIL;
	IWrapperSoapDeSerializer* pIWSDZ = NULL;
	mc->getSoapDeSerializer(&pIWSDZ);
	if (!pIWSDZ) return AXIS_FAIL;
	pIWSSZ->createSoapMethod("AddPointResponse", pIWSSZ->getNewNamespacePrefix(), "http://www.opensource.lk/CPPService");

	Point* v0 = (Point*)pIWSDZ->GetObject((void*)Axis_DeSerialize_Point, 
		(void*)Axis_Create_Point, (void*)Axis_Delete_Point, 
		Axis_TypeName_Point, Axis_URI_Point);

	Point* v1 = (Point*)pIWSDZ->GetObject((void*)Axis_DeSerialize_Point, 
		(void*)Axis_Create_Point, (void*)Axis_Delete_Point, 
		Axis_TypeName_Point, Axis_URI_Point);

	if (AXIS_SUCCESS != pIWSDZ->GetStatus())
	{
		Axis_Delete_Point(v0);
		Axis_Delete_Point(v1);
		return AXIS_DESERIALIZATION_ERROR;
	}
	//Call actual web service method with appropriate parameters
	Point* ret = AddPoint(v0, v1);

	return pIWSSZ->AddOutputParam("AddPointReturn", ret, (void*)Axis_Serialize_Point, (void*)Axis_Delete_Point);
}

int CServiceWrapper::AddWrapped(IMessageData* mc)
{
	IWrapperSoapSerializer* pIWSSZ = NULL;
	mc->getSoapSerializer(&pIWSSZ);
	if (!pIWSSZ) return AXIS_FAIL;
	IWrapperSoapDeSerializer* pIWSDZ = NULL;
	mc->getSoapDeSerializer(&pIWSDZ);
	if (!pIWSDZ) return AXIS_FAIL;
	pIWSSZ->createSoapMethod("AddResponse", pIWSSZ->getNewNamespacePrefix(), "http://www.opensource.lk/CPPService");

	int v0 = pIWSDZ->GetInt();

	int v1 = pIWSDZ->GetInt();

	//Call actual web service method with appropriate parameters
	int ret = AddInt(v0, v1);

	return pIWSSZ->AddOutputParam("AddReturn", ret, XSD_INT);
}

int CServiceWrapper::PerimeterWrapped(IMessageData* mc)
{
	IWrapperSoapSerializer* pIWSSZ = NULL;
	mc->getSoapSerializer(&pIWSSZ);
	if (!pIWSSZ) return AXIS_FAIL;
	IWrapperSoapDeSerializer* pIWSDZ = NULL;
	mc->getSoapDeSerializer(&pIWSDZ);
	if (!pIWSDZ) return AXIS_FAIL;
	pIWSSZ->createSoapMethod("PerimeterResponse", pIWSSZ->getNewNamespacePrefix(), "http://www.opensource.lk/CPPService");

	Triangle* v0 = (Triangle*)pIWSDZ->GetObject((void*)Axis_DeSerialize_Triangle, 
		(void*)Axis_Create_Triangle, (void*)Axis_Delete_Triangle, 
		Axis_TypeName_Triangle, Axis_URI_Triangle);

	if (AXIS_SUCCESS != pIWSDZ->GetStatus())
	{
		Axis_Delete_Triangle(v0);
		return AXIS_DESERIALIZATION_ERROR;
	}
	//Call actual web service method with appropriate parameters
	double ret = Perimeter(v0);

	return pIWSSZ->AddOutputParam("PerimeterReturn", ret, XSD_DOUBLE);
}
