/* -*- C++ -*- */

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "SOAP" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 *
 *
 *
 * @author Roshan Weerasuriya (roshan@jkcs.slt.lk, roshan@opensource.lk)
 *
 */

// CPP_DeploymentWrapper.cpp: implementation of the CPP_DeploymentWrapper class.
//
//////////////////////////////////////////////////////////////////////

#include "CPP_DeploymentWrapper.h"
#include <axis/common/IDeployerUtils.h>
#ifdef _DEBUG
	#include <axis/common/AxisTrace.h>
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CPP_DeploymentWrapper::CPP_DeploymentWrapper()
{
	//pWs = new CPP_Deployment();
}

CPP_DeploymentWrapper::~CPP_DeploymentWrapper()
{

}

int CPP_DeploymentWrapper::Invoke(IMessageData *mc)
{
	IWrapperSoapDeSerializer* pIWSDZ = NULL;
	mc->getSoapDeSerializer(&pIWSDZ);
	if (!pIWSDZ) return AXIS_FAIL;
	const AxisChar* method = pIWSDZ->GetMethodName();
	if (0 == strcmp(method, "deploy"))
		return deploy(mc);
	else if (0 == strcmp(method, "undeploy"))
		return unDeploy(mc);
	else return AXIS_FAIL; //CF_METHOD_NOT_FOUND_IN_LIBRARY
}

void CPP_DeploymentWrapper::OnFault(IMessageData *pMsg)
{

}

int CPP_DeploymentWrapper::Init()
{
	return AXIS_SUCCESS;
}

int CPP_DeploymentWrapper::Fini()
{
	return AXIS_SUCCESS;
}

int CPP_DeploymentWrapper::deploy(IMessageData *pIMessageData)
{
	#ifdef _DEBUG
		AXISTRACE3("start of the wrapper deploy method");
	#endif

	IWrapperSoapSerializer* pIWSSZ = NULL;
	pIMessageData->getSoapSerializer(&pIWSSZ);
	if (!pIWSSZ) return AXIS_FAIL;
	IWrapperSoapDeSerializer* pIWSDZ = NULL;
	pIMessageData->getSoapDeSerializer(&pIWSDZ);
	if (!pIWSDZ) return AXIS_FAIL;

	pIWSSZ->createSoapMethod("deployResponse", pIWSSZ->getNewNamespacePrefix(), "http://www.opensource.lk/CPP_Deployment");

	string asServiceName = pIWSDZ->GetString();

	string asDllPath = pIWSDZ->GetString();
	
	IParam *param2 = pIWSDZ->GetParam(); // we know that this is an array
	Axis_ArrayTag inArray;
	param2->GetArraySize(&(inArray.m_Size));
	if (inArray.m_Size > 0)
		inArray.m_Array = new string[inArray.m_Size];
	else 
		return AXIS_FAIL; //CF_ZERO_ARRAY_SIZE_ERROR
	
	param2->SetArrayElements((void*)(inArray.m_Array));
	pIWSDZ->Deserialize(param2,0);		
	
	int iResult = 1;

	//int iResult = pWs->deploy(asServiceName, asDllPath, inArray, iDepoy);
	IDeployerUtils* pIDeployerUtils;
	pIMessageData->getWSDDDeployment(&pIDeployerUtils);
	pIDeployerUtils->deploy(asServiceName, asDllPath, inArray);

	return pIWSSZ->AddOutputParam("deployReturn", iResult, XSD_INT);

	return AXIS_SUCCESS;
}

int CPP_DeploymentWrapper::unDeploy(IMessageData *pIMessageData)
{
	#ifdef _DEBUG
		AXISTRACE3("start of the wrapper unDeploy method");
	#endif

	IWrapperSoapSerializer* pIWSSZ = NULL;
	pIMessageData->getSoapSerializer(&pIWSSZ);
	if (!pIWSSZ) return AXIS_FAIL;
	IWrapperSoapDeSerializer* pIWSDZ = NULL;
	pIMessageData->getSoapDeSerializer(&pIWSDZ);
	if (!pIWSDZ) return AXIS_FAIL;

	pIWSSZ->createSoapMethod("unDeployResponse", pIWSSZ->getNewNamespacePrefix(), "http://www.opensource.lk/CPP_Deployment");

	string asServiceName = pIWSDZ->GetString();
	
	int iResult = 1;

	//int iResult = pWs->deploy(asServiceName, asDllPath, inArray, iDepoy);
	IDeployerUtils* pIDeployerUtils;
	pIMessageData->getWSDDDeployment(&pIDeployerUtils);
	pIDeployerUtils->unDeploy(asServiceName);

	return pIWSSZ->AddOutputParam("deployReturn", iResult, XSD_INT);

	return AXIS_SUCCESS;
}
